/* input-output.c generated by valac 0.56.17, the Vala compiler
 * generated from input-output.vala, do not modify */

/*   FILE: io.vala -- Handle PiFace CAD or keyboard I/O.
 * AUTHOR: W. Michael Petullo <mike@flyn.org>
 *   DATE: 22 February 2018
 *
 * Copyright (c) 2018 W. Michael Petullo <new@flyn.org>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* The PiFace CAD provides the following inputs:
 *   From left to right:
 *     Button 0 = 0
 *     Button 1 = 1
 *     Button 2 = 2
 *     Button 3 = 3
 *     Button 4 = 4
 *   Rocker switch:
 *     Select     = 5
 *     Rock left  = 6
 *     Rock right = 7
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <pifacecad.h>
#include <stdio.h>
#include <unistd.h>

#define LIGHTS_OUT_SECONDS ((guint) 10)
#define SWITCH_POLL_MILLISECONDS ((guint) 20)
#define OUTPUT_REFRESH_FREQUENCY ((guint) 750)
#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif
#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

typedef enum  {
	SWITCH_NUMBER_STOP = 0,
	SWITCH_NUMBER_SEEKBACK = 1,
	SWITCH_NUMBER_PAUSEPLAY = 2,
	SWITCH_NUMBER_SEEKFORWARD = 3,
	SWITCH_NUMBER_ESCAPE = 4,
	SWITCH_NUMBER_SELECT = 5,
	SWITCH_NUMBER_LEFT = 6,
	SWITCH_NUMBER_RIGHT = 7
} SwitchNumber;

#define TYPE_SWITCH_NUMBER (switch_number_get_type ())
typedef enum  {
	KEY_NUMBER_STOP = 'z',
	KEY_NUMBER_SEEKBACK = 'q',
	KEY_NUMBER_PAUSEPLAY = ' ',
	KEY_NUMBER_SEEKFORWARD = 'w',
	KEY_NUMBER_ESCAPE = 0x1b,
	KEY_NUMBER_SELECT = '\n',
	KEY_NUMBER_LEFT = 'a',
	KEY_NUMBER_RIGHT = 's'
} KeyNumber;

#define TYPE_KEY_NUMBER (key_number_get_type ())

#define TYPE_INPUT_OUTPUT (input_output_get_type ())
#define INPUT_OUTPUT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_INPUT_OUTPUT, InputOutput))
#define INPUT_OUTPUT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_INPUT_OUTPUT, InputOutputClass))
#define IS_INPUT_OUTPUT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_INPUT_OUTPUT))
#define IS_INPUT_OUTPUT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_INPUT_OUTPUT))
#define INPUT_OUTPUT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_INPUT_OUTPUT, InputOutputClass))

typedef struct _InputOutput InputOutput;
typedef struct _InputOutputClass InputOutputClass;
typedef struct _InputOutputPrivate InputOutputPrivate;
enum  {
	INPUT_OUTPUT_0_PROPERTY,
	INPUT_OUTPUT_NUM_PROPERTIES
};
static GParamSpec* input_output_properties[INPUT_OUTPUT_NUM_PROPERTIES];
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_string_free0(var) ((var == NULL) ? NULL : (var = (g_string_free (var, TRUE), NULL)))
#define _g_io_channel_unref0(var) ((var == NULL) ? NULL : (var = (g_io_channel_unref (var), NULL)))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
enum  {
	INPUT_OUTPUT_STOP_SIGNAL,
	INPUT_OUTPUT_SEEKBACK_SIGNAL,
	INPUT_OUTPUT_PAUSEPLAY_SIGNAL,
	INPUT_OUTPUT_SEEKFORWARD_SIGNAL,
	INPUT_OUTPUT_ESCAPE_SIGNAL,
	INPUT_OUTPUT_LEFT_SIGNAL,
	INPUT_OUTPUT_RIGHT_SIGNAL,
	INPUT_OUTPUT_SELECT_SIGNAL,
	INPUT_OUTPUT_NUM_SIGNALS
};
static guint input_output_signals[INPUT_OUTPUT_NUM_SIGNALS] = {0};
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _InputOutput {
	GObject parent_instance;
	InputOutputPrivate * priv;
};

struct _InputOutputClass {
	GObjectClass parent_class;
};

struct _InputOutputPrivate {
	gboolean pifacecad;
	guint lights_out_id;
	guint8* lastSwitchState;
	gint lastSwitchState_length1;
	gint _lastSwitchState_size_;
};

static gint InputOutput_private_offset;
static gpointer input_output_parent_class = NULL;
static guint input_output_output_shift;
static guint input_output_output_shift = (guint) 0;
static gchar* input_output_output_str;
static gchar* input_output_output_str = NULL;
VALA_EXTERN gboolean no_pifacecad;

VALA_EXTERN GType switch_number_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType key_number_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType input_output_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (InputOutput, g_object_unref)
VALA_EXTERN InputOutput* input_output_new (gboolean pifacecad);
VALA_EXTERN InputOutput* input_output_construct (GType object_type,
                                     gboolean pifacecad);
static gboolean input_output_render_output (InputOutput* self);
static gboolean _input_output_render_output_gsource_func (gpointer self);
VALA_EXTERN void input_output_register_handlers (InputOutput* self);
static void input_output_lights (InputOutput* self);
VALA_EXTERN void input_output_output (InputOutput* self,
                          const gchar* str);
VALA_EXTERN gboolean input_output_check_keyboard (InputOutput* self,
                                      GIOChannel* channel,
                                      GIOCondition condition);
static gboolean _input_output_check_keyboard_gio_func (GIOChannel* source,
                                                GIOCondition condition,
                                                gpointer self);
VALA_EXTERN gboolean input_output_check_switches (InputOutput* self);
static gboolean _input_output_check_switches_gsource_func (gpointer self);
static gboolean ___lambda4_ (InputOutput* self);
static gboolean ____lambda4__gsource_func (gpointer self);
static void input_output_finalize (GObject * obj);
static GType input_output_get_type_once (void);

static GType
switch_number_get_type_once (void)
{
	static const GEnumValue values[] = {{SWITCH_NUMBER_STOP, "SWITCH_NUMBER_STOP", "stop"}, {SWITCH_NUMBER_SEEKBACK, "SWITCH_NUMBER_SEEKBACK", "seekback"}, {SWITCH_NUMBER_PAUSEPLAY, "SWITCH_NUMBER_PAUSEPLAY", "pauseplay"}, {SWITCH_NUMBER_SEEKFORWARD, "SWITCH_NUMBER_SEEKFORWARD", "seekforward"}, {SWITCH_NUMBER_ESCAPE, "SWITCH_NUMBER_ESCAPE", "escape"}, {SWITCH_NUMBER_SELECT, "SWITCH_NUMBER_SELECT", "select"}, {SWITCH_NUMBER_LEFT, "SWITCH_NUMBER_LEFT", "left"}, {SWITCH_NUMBER_RIGHT, "SWITCH_NUMBER_RIGHT", "right"}, {0, NULL, NULL}};
	GType switch_number_type_id;
	switch_number_type_id = g_enum_register_static ("SwitchNumber", values);
	return switch_number_type_id;
}

GType
switch_number_get_type (void)
{
	static volatile gsize switch_number_type_id__once = 0;
	if (g_once_init_enter (&switch_number_type_id__once)) {
		GType switch_number_type_id;
		switch_number_type_id = switch_number_get_type_once ();
		g_once_init_leave (&switch_number_type_id__once, switch_number_type_id);
	}
	return switch_number_type_id__once;
}

static GType
key_number_get_type_once (void)
{
	static const GEnumValue values[] = {{KEY_NUMBER_STOP, "KEY_NUMBER_STOP", "stop"}, {KEY_NUMBER_SEEKBACK, "KEY_NUMBER_SEEKBACK", "seekback"}, {KEY_NUMBER_PAUSEPLAY, "KEY_NUMBER_PAUSEPLAY", "pauseplay"}, {KEY_NUMBER_SEEKFORWARD, "KEY_NUMBER_SEEKFORWARD", "seekforward"}, {KEY_NUMBER_ESCAPE, "KEY_NUMBER_ESCAPE", "escape"}, {KEY_NUMBER_SELECT, "KEY_NUMBER_SELECT", "select"}, {KEY_NUMBER_LEFT, "KEY_NUMBER_LEFT", "left"}, {KEY_NUMBER_RIGHT, "KEY_NUMBER_RIGHT", "right"}, {0, NULL, NULL}};
	GType key_number_type_id;
	key_number_type_id = g_enum_register_static ("KeyNumber", values);
	return key_number_type_id;
}

GType
key_number_get_type (void)
{
	static volatile gsize key_number_type_id__once = 0;
	if (g_once_init_enter (&key_number_type_id__once)) {
		GType key_number_type_id;
		key_number_type_id = key_number_get_type_once ();
		g_once_init_leave (&key_number_type_id__once, key_number_type_id);
	}
	return key_number_type_id__once;
}

static inline gpointer
input_output_get_instance_private (InputOutput* self)
{
	return G_STRUCT_MEMBER_P (self, InputOutput_private_offset);
}

static gboolean
_input_output_render_output_gsource_func (gpointer self)
{
	gboolean result;
	result = input_output_render_output ((InputOutput*) self);
	return result;
}

InputOutput*
input_output_construct (GType object_type,
                        gboolean pifacecad)
{
	InputOutput * self = NULL;
	self = (InputOutput*) g_object_new (object_type, NULL);
	self->priv->pifacecad = pifacecad;
	if (pifacecad) {
		pifacecad_open ();
		g_timeout_add_full (G_PRIORITY_DEFAULT, (guint) 750, _input_output_render_output_gsource_func, g_object_ref (self), g_object_unref);
	}
	input_output_register_handlers (self);
	input_output_lights (self);
	return self;
}

InputOutput*
input_output_new (gboolean pifacecad)
{
	return input_output_construct (TYPE_INPUT_OUTPUT, pifacecad);
}

static glong
string_strnlen (gchar* str,
                glong maxlen)
{
	gchar* end = NULL;
	gchar* _tmp0_;
	gchar* _tmp1_;
	glong result;
	_tmp0_ = memchr (str, 0, (gsize) maxlen);
	end = _tmp0_;
	_tmp1_ = end;
	if (_tmp1_ == NULL) {
		result = maxlen;
		return result;
	} else {
		gchar* _tmp2_;
		_tmp2_ = end;
		result = (glong) (_tmp2_ - str);
		return result;
	}
}

static gchar*
string_substring (const gchar* self,
                  glong offset,
                  glong len)
{
	glong string_length = 0L;
	gboolean _tmp0_ = FALSE;
	gchar* _tmp3_;
	gchar* result;
	g_return_val_if_fail (self != NULL, NULL);
	if (offset >= ((glong) 0)) {
		_tmp0_ = len >= ((glong) 0);
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		string_length = string_strnlen ((gchar*) self, offset + len);
	} else {
		gint _tmp1_;
		gint _tmp2_;
		_tmp1_ = strlen (self);
		_tmp2_ = _tmp1_;
		string_length = (glong) _tmp2_;
	}
	if (offset < ((glong) 0)) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= ((glong) 0), NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < ((glong) 0)) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	_tmp3_ = g_strndup (((gchar*) self) + offset, (gsize) len);
	result = _tmp3_;
	return result;
}

static gboolean
input_output_render_output (InputOutput* self)
{
	const gchar* _tmp0_;
	gint _tmp1_;
	gint _tmp2_;
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	pifacecad_lcd_clear ();
	_tmp0_ = input_output_output_str;
	_tmp1_ = strlen (_tmp0_);
	_tmp2_ = _tmp1_;
	if (_tmp2_ <= 15) {
		const gchar* _tmp3_;
		_tmp3_ = input_output_output_str;
		pifacecad_lcd_write (_tmp3_);
	} else {
		GString* builder = NULL;
		GString* _tmp4_;
		gint _tmp5_ = 0;
		const gchar* _tmp6_;
		gint _tmp7_;
		gint _tmp8_;
		guint rest = 0U;
		gint _tmp12_ = 0;
		const gchar* _tmp13_;
		gint _tmp14_;
		gint _tmp15_;
		guint cutoff = 0U;
		GString* _tmp19_;
		const gchar* _tmp20_;
		gchar* _tmp21_;
		gchar* _tmp22_;
		GString* _tmp23_;
		GString* _tmp24_;
		const gchar* _tmp25_;
		gchar* _tmp26_;
		gchar* _tmp27_;
		GString* _tmp28_;
		const gchar* _tmp29_;
		guint _tmp30_ = 0U;
		_tmp4_ = g_string_new ("");
		builder = _tmp4_;
		_tmp6_ = input_output_output_str;
		_tmp7_ = strlen (_tmp6_);
		_tmp8_ = _tmp7_;
		if (_tmp8_ > 30) {
			_tmp5_ = 15;
		} else {
			const gchar* _tmp9_;
			gint _tmp10_;
			gint _tmp11_;
			_tmp9_ = input_output_output_str;
			_tmp10_ = strlen (_tmp9_);
			_tmp11_ = _tmp10_;
			_tmp5_ = _tmp11_ - 15;
		}
		rest = (guint) _tmp5_;
		_tmp13_ = input_output_output_str;
		_tmp14_ = strlen (_tmp13_);
		_tmp15_ = _tmp14_;
		if (_tmp15_ > 30) {
			const gchar* _tmp16_;
			gint _tmp17_;
			gint _tmp18_;
			_tmp16_ = input_output_output_str;
			_tmp17_ = strlen (_tmp16_);
			_tmp18_ = _tmp17_;
			_tmp12_ = _tmp18_ - 30;
		} else {
			_tmp12_ = 0;
		}
		cutoff = (guint) _tmp12_;
		_tmp19_ = builder;
		_tmp20_ = input_output_output_str;
		_tmp21_ = string_substring (_tmp20_, (glong) input_output_output_shift, (glong) 15);
		_tmp22_ = _tmp21_;
		g_string_append (_tmp19_, _tmp22_);
		_g_free0 (_tmp22_);
		_tmp23_ = builder;
		g_string_append (_tmp23_, "\n ");
		_tmp24_ = builder;
		_tmp25_ = input_output_output_str;
		_tmp26_ = string_substring (_tmp25_, (glong) (15 + input_output_output_shift), (glong) rest);
		_tmp27_ = _tmp26_;
		g_string_append (_tmp24_, _tmp27_);
		_g_free0 (_tmp27_);
		_tmp28_ = builder;
		_tmp29_ = _tmp28_->str;
		pifacecad_lcd_write (_tmp29_);
		if (input_output_output_shift < cutoff) {
			_tmp30_ = input_output_output_shift + 1;
		} else {
			_tmp30_ = (guint) 0;
		}
		input_output_output_shift = _tmp30_;
		_g_string_free0 (builder);
	}
	result = TRUE;
	return result;
}

void
input_output_output (InputOutput* self,
                     const gchar* str)
{
	FILE* _tmp0_;
	gchar* _tmp1_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (str != NULL);
	_tmp0_ = stdout;
	fprintf (_tmp0_, "%s\n", str);
	input_output_output_shift = (guint) 0;
	_tmp1_ = g_strdup (str);
	_g_free0 (input_output_output_str);
	input_output_output_str = _tmp1_;
	if (!no_pifacecad) {
		input_output_render_output (self);
	}
}

static gboolean
_input_output_check_keyboard_gio_func (GIOChannel* source,
                                       GIOCondition condition,
                                       gpointer self)
{
	gboolean result;
	result = input_output_check_keyboard ((InputOutput*) self, source, condition);
	return result;
}

static gboolean
_input_output_check_switches_gsource_func (gpointer self)
{
	gboolean result;
	result = input_output_check_switches ((InputOutput*) self);
	return result;
}

void
input_output_register_handlers (InputOutput* self)
{
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	{
		GIOChannel* channel = NULL;
		GIOChannel* _tmp0_;
		_tmp0_ = g_io_channel_unix_new (STDIN_FILENO);
		channel = _tmp0_;
		g_io_channel_set_encoding (channel, NULL, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			_g_io_channel_unref0 (channel);
			if (_inner_error0_->domain == G_IO_CHANNEL_ERROR) {
				goto __catch0_g_io_channel_error;
			}
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
		g_io_add_watch (channel, G_IO_IN, _input_output_check_keyboard_gio_func, self);
		if (self->priv->pifacecad) {
			g_timeout_add_full (G_PRIORITY_DEFAULT, SWITCH_POLL_MILLISECONDS, _input_output_check_switches_gsource_func, g_object_ref (self), g_object_unref);
		}
		_g_io_channel_unref0 (channel);
	}
	goto __finally0;
	__catch0_g_io_channel_error:
	{
		g_clear_error (&_inner_error0_);
		g_error ("input-output.vala:138: Channel error initializing state");
	}
	__finally0:
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
		g_clear_error (&_inner_error0_);
		return;
	}
}

static gboolean
___lambda4_ (InputOutput* self)
{
	gboolean result;
	pifacecad_lcd_backlight_off ();
	result = FALSE;
	return result;
}

static gboolean
____lambda4__gsource_func (gpointer self)
{
	gboolean result;
	result = ___lambda4_ ((InputOutput*) self);
	return result;
}

static void
input_output_lights (InputOutput* self)
{
	g_return_if_fail (self != NULL);
	if (self->priv->pifacecad) {
		pifacecad_lcd_backlight_on ();
		if (((guint) 0) != self->priv->lights_out_id) {
			g_source_remove (self->priv->lights_out_id);
		}
		self->priv->lights_out_id = g_timeout_add_seconds_full (G_PRIORITY_DEFAULT, LIGHTS_OUT_SECONDS, ____lambda4__gsource_func, g_object_ref (self), g_object_unref);
	}
}

gboolean
input_output_check_switches (InputOutput* self)
{
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	_vala_assert (self->priv->pifacecad, "pifacecad");
	{
		guint8 _switch_ = 0U;
		_switch_ = (guint8) 0;
		{
			gboolean _tmp0_ = FALSE;
			_tmp0_ = TRUE;
			while (TRUE) {
				guint8 current = 0U;
				gboolean _tmp2_ = FALSE;
				guint8* _tmp5_;
				gint _tmp5__length1;
				if (!_tmp0_) {
					guint8 _tmp1_;
					_tmp1_ = _switch_;
					_switch_ = _tmp1_ + 1;
				}
				_tmp0_ = FALSE;
				if (!(((gint) _switch_) < 8)) {
					break;
				}
				current = pifacecad_read_switch (_switch_);
				if (0 == ((gint) current)) {
					guint8* _tmp3_;
					gint _tmp3__length1;
					guint8 _tmp4_;
					_tmp3_ = self->priv->lastSwitchState;
					_tmp3__length1 = self->priv->lastSwitchState_length1;
					_tmp4_ = _tmp3_[_switch_];
					_tmp2_ = 1 == ((gint) _tmp4_);
				} else {
					_tmp2_ = FALSE;
				}
				if (_tmp2_) {
					g_debug ("input-output.vala:163: Switch %d input\n", (gint) _switch_);
					switch (_switch_) {
						case SWITCH_NUMBER_STOP:
						{
							g_signal_emit (self, input_output_signals[INPUT_OUTPUT_STOP_SIGNAL], 0);
							break;
						}
						case SWITCH_NUMBER_SEEKBACK:
						{
							g_signal_emit (self, input_output_signals[INPUT_OUTPUT_SEEKBACK_SIGNAL], 0);
							break;
						}
						case SWITCH_NUMBER_PAUSEPLAY:
						{
							g_signal_emit (self, input_output_signals[INPUT_OUTPUT_PAUSEPLAY_SIGNAL], 0);
							break;
						}
						case SWITCH_NUMBER_SEEKFORWARD:
						{
							g_signal_emit (self, input_output_signals[INPUT_OUTPUT_SEEKFORWARD_SIGNAL], 0);
							break;
						}
						case SWITCH_NUMBER_ESCAPE:
						{
							g_signal_emit (self, input_output_signals[INPUT_OUTPUT_ESCAPE_SIGNAL], 0);
							break;
						}
						case SWITCH_NUMBER_SELECT:
						{
							g_signal_emit (self, input_output_signals[INPUT_OUTPUT_SELECT_SIGNAL], 0);
							break;
						}
						case SWITCH_NUMBER_LEFT:
						{
							g_signal_emit (self, input_output_signals[INPUT_OUTPUT_LEFT_SIGNAL], 0);
							break;
						}
						case SWITCH_NUMBER_RIGHT:
						{
							g_signal_emit (self, input_output_signals[INPUT_OUTPUT_RIGHT_SIGNAL], 0);
							break;
						}
						default:
						{
							g_warning ("input-output.vala:191: Invalid key pressed");
							break;
						}
					}
					input_output_lights (self);
				}
				_tmp5_ = self->priv->lastSwitchState;
				_tmp5__length1 = self->priv->lastSwitchState_length1;
				_tmp5_[_switch_] = current;
			}
		}
	}
	result = TRUE;
	return result;
}

gboolean
input_output_check_keyboard (InputOutput* self,
                             GIOChannel* channel,
                             GIOCondition condition)
{
	gchar* c = NULL;
	gchar* _tmp0_;
	gint c_length1;
	gint _c_size_;
	gchar* _tmp15_;
	gint _tmp15__length1;
	gchar _tmp16_;
	GError* _inner_error0_ = NULL;
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (channel != NULL, FALSE);
	_tmp0_ = g_new0 (gchar, 1);
	c = _tmp0_;
	c_length1 = 1;
	_c_size_ = c_length1;
	if (condition == G_IO_HUP) {
		FILE* _tmp1_;
		_tmp1_ = stderr;
		fprintf (_tmp1_, "Standard in closed?\n");
		result = FALSE;
		c = (g_free (c), NULL);
		return result;
	}
	{
		gsize length = 0UL;
		GIOStatus status = 0;
		gchar* _tmp2_;
		gint _tmp2__length1;
		gsize _tmp3_ = 0UL;
		GIOStatus _tmp4_;
		gboolean _tmp6_ = FALSE;
		length = (gsize) -1;
		_tmp2_ = c;
		_tmp2__length1 = c_length1;
		_tmp4_ = g_io_channel_read_chars (channel, _tmp2_, (gint) _tmp2__length1, &_tmp3_, &_inner_error0_);
		length = _tmp3_;
		status = _tmp4_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			gboolean _tmp5_ = FALSE;
			if (_inner_error0_->domain == G_IO_CHANNEL_ERROR) {
				goto __catch0_g_io_channel_error;
			}
			if (_inner_error0_->domain == G_CONVERT_ERROR) {
				goto __catch0_g_convert_error;
			}
			c = (g_free (c), NULL);
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return _tmp5_;
		}
		if (status == G_IO_STATUS_EOF) {
			_tmp6_ = TRUE;
		} else {
			_tmp6_ = length != ((gsize) 1);
		}
		if (_tmp6_) {
			FILE* _tmp7_;
			_tmp7_ = stderr;
			fprintf (_tmp7_, "Standard in closed?\n");
			result = FALSE;
			c = (g_free (c), NULL);
			return result;
		}
	}
	goto __finally0;
	__catch0_g_io_channel_error:
	{
		GError* e = NULL;
		FILE* _tmp8_;
		GError* _tmp9_;
		const gchar* _tmp10_;
		e = _inner_error0_;
		_inner_error0_ = NULL;
		_tmp8_ = stderr;
		_tmp9_ = e;
		_tmp10_ = _tmp9_->message;
		fprintf (_tmp8_, "IOChannelError: %s\n", _tmp10_);
		result = FALSE;
		_g_error_free0 (e);
		c = (g_free (c), NULL);
		return result;
	}
	goto __finally0;
	__catch0_g_convert_error:
	{
		GError* e = NULL;
		FILE* _tmp11_;
		GError* _tmp12_;
		const gchar* _tmp13_;
		e = _inner_error0_;
		_inner_error0_ = NULL;
		_tmp11_ = stderr;
		_tmp12_ = e;
		_tmp13_ = _tmp12_->message;
		fprintf (_tmp11_, "ConvertError: %s\n", _tmp13_);
		result = FALSE;
		_g_error_free0 (e);
		c = (g_free (c), NULL);
		return result;
	}
	__finally0:
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		gboolean _tmp14_ = FALSE;
		c = (g_free (c), NULL);
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
		g_clear_error (&_inner_error0_);
		return _tmp14_;
	}
	_tmp15_ = c;
	_tmp15__length1 = c_length1;
	_tmp16_ = _tmp15_[0];
	switch (_tmp16_) {
		case KEY_NUMBER_STOP:
		{
			g_signal_emit (self, input_output_signals[INPUT_OUTPUT_STOP_SIGNAL], 0);
			break;
		}
		case KEY_NUMBER_SEEKBACK:
		{
			g_signal_emit (self, input_output_signals[INPUT_OUTPUT_SEEKBACK_SIGNAL], 0);
			break;
		}
		case KEY_NUMBER_SEEKFORWARD:
		{
			g_signal_emit (self, input_output_signals[INPUT_OUTPUT_SEEKFORWARD_SIGNAL], 0);
			break;
		}
		case KEY_NUMBER_PAUSEPLAY:
		{
			g_signal_emit (self, input_output_signals[INPUT_OUTPUT_PAUSEPLAY_SIGNAL], 0);
			break;
		}
		case KEY_NUMBER_LEFT:
		{
			g_signal_emit (self, input_output_signals[INPUT_OUTPUT_LEFT_SIGNAL], 0);
			break;
		}
		case KEY_NUMBER_RIGHT:
		{
			g_signal_emit (self, input_output_signals[INPUT_OUTPUT_RIGHT_SIGNAL], 0);
			break;
		}
		case KEY_NUMBER_SELECT:
		{
			g_signal_emit (self, input_output_signals[INPUT_OUTPUT_SELECT_SIGNAL], 0);
			break;
		}
		case KEY_NUMBER_ESCAPE:
		{
			g_signal_emit (self, input_output_signals[INPUT_OUTPUT_ESCAPE_SIGNAL], 0);
			break;
		}
		default:
		{
			FILE* _tmp17_;
			FILE* _tmp18_;
			FILE* _tmp19_;
			FILE* _tmp20_;
			FILE* _tmp21_;
			FILE* _tmp22_;
			FILE* _tmp23_;
			FILE* _tmp24_;
			FILE* _tmp25_;
			_tmp17_ = stderr;
			fprintf (_tmp17_, "Valid keys are:\n");
			_tmp18_ = stderr;
			fprintf (_tmp18_, "  'z'     (stop)\n");
			_tmp19_ = stderr;
			fprintf (_tmp19_, "  'q'     (seek backward)\n");
			_tmp20_ = stderr;
			fprintf (_tmp20_, "  'w'     (seek forward)\n");
			_tmp21_ = stderr;
			fprintf (_tmp21_, "  ' '     (pause/play)\n");
			_tmp22_ = stderr;
			fprintf (_tmp22_, "  'a'     (left)\n");
			_tmp23_ = stderr;
			fprintf (_tmp23_, "  's'     (right)\n");
			_tmp24_ = stderr;
			fprintf (_tmp24_, "  'enter' (select)\n");
			_tmp25_ = stderr;
			fprintf (_tmp25_, "  'esc'   (escape)\n");
			break;
		}
	}
	result = TRUE;
	c = (g_free (c), NULL);
	return result;
}

static void
input_output_class_init (InputOutputClass * klass,
                         gpointer klass_data)
{
	gchar* _tmp0_;
	input_output_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &InputOutput_private_offset);
	G_OBJECT_CLASS (klass)->finalize = input_output_finalize;
	input_output_signals[INPUT_OUTPUT_STOP_SIGNAL] = g_signal_new ("stop", TYPE_INPUT_OUTPUT, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	input_output_signals[INPUT_OUTPUT_SEEKBACK_SIGNAL] = g_signal_new ("seekback", TYPE_INPUT_OUTPUT, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	input_output_signals[INPUT_OUTPUT_PAUSEPLAY_SIGNAL] = g_signal_new ("pauseplay", TYPE_INPUT_OUTPUT, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	input_output_signals[INPUT_OUTPUT_SEEKFORWARD_SIGNAL] = g_signal_new ("seekforward", TYPE_INPUT_OUTPUT, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	input_output_signals[INPUT_OUTPUT_ESCAPE_SIGNAL] = g_signal_new ("escape", TYPE_INPUT_OUTPUT, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	input_output_signals[INPUT_OUTPUT_LEFT_SIGNAL] = g_signal_new ("left", TYPE_INPUT_OUTPUT, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	input_output_signals[INPUT_OUTPUT_RIGHT_SIGNAL] = g_signal_new ("right", TYPE_INPUT_OUTPUT, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	input_output_signals[INPUT_OUTPUT_SELECT_SIGNAL] = g_signal_new ("select", TYPE_INPUT_OUTPUT, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	_tmp0_ = g_strdup ("");
	input_output_output_str = _tmp0_;
}

static void
input_output_instance_init (InputOutput * self,
                            gpointer klass)
{
	guint8* _tmp0_;
	self->priv = input_output_get_instance_private (self);
	self->priv->lights_out_id = (guint) 0;
	_tmp0_ = g_new0 (guint8, 8);
	_tmp0_[0] = (guint8) 1;
	_tmp0_[1] = (guint8) 1;
	_tmp0_[2] = (guint8) 1;
	_tmp0_[3] = (guint8) 1;
	_tmp0_[4] = (guint8) 1;
	_tmp0_[5] = (guint8) 1;
	_tmp0_[6] = (guint8) 1;
	_tmp0_[7] = (guint8) 1;
	self->priv->lastSwitchState = _tmp0_;
	self->priv->lastSwitchState_length1 = 8;
	self->priv->_lastSwitchState_size_ = self->priv->lastSwitchState_length1;
}

static void
input_output_finalize (GObject * obj)
{
	InputOutput * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_INPUT_OUTPUT, InputOutput);
	self->priv->lastSwitchState = (g_free (self->priv->lastSwitchState), NULL);
	G_OBJECT_CLASS (input_output_parent_class)->finalize (obj);
}

static GType
input_output_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (InputOutputClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) input_output_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (InputOutput), 0, (GInstanceInitFunc) input_output_instance_init, NULL };
	GType input_output_type_id;
	input_output_type_id = g_type_register_static (G_TYPE_OBJECT, "InputOutput", &g_define_type_info, 0);
	InputOutput_private_offset = g_type_add_instance_private (input_output_type_id, sizeof (InputOutputPrivate));
	return input_output_type_id;
}

GType
input_output_get_type (void)
{
	static volatile gsize input_output_type_id__once = 0;
	if (g_once_init_enter (&input_output_type_id__once)) {
		GType input_output_type_id;
		input_output_type_id = input_output_get_type_once ();
		g_once_init_leave (&input_output_type_id__once, input_output_type_id);
	}
	return input_output_type_id__once;
}

